@@ -21,23 +21,24 @@
  *
  */
 
-/** @var $this OC\Route\Router */
+namespace OCA\Activity\AppInfo;
 
-$this->create('activity.index', '')
-	->actionInclude('activity/index.php');
-$this->create('activity.ajax.fetch', 'ajax/fetch.php')
-	->actionInclude('activity/ajax/fetch.php');
-$this->create('activity.ajax.rssfeed', 'ajax/rssfeed.php')
-	->actionInclude('activity/ajax/rssfeed.php');
-$this->create('activity.ajax.settings', 'ajax/settings.php')
-	->actionInclude('activity/ajax/settings.php');
+/** @var $this \OC\Route\Router */
 $this->create('activity.rss', 'rss.php')
 	->actionInclude('activity/rss.php');
 
 // Register an OCS API call
-OC_API::register(
+\OC_API::register(
 	'get',
 	'/cloud/activity',
 	array('OCA\Activity\Api', 'get'),
 	'activity'
 );
+
+$application = new Application();
+$application->registerRoutes($this, ['routes' => [
+	['name' => 'Settings#personal', 'url' => '/settings', 'verb' => 'POST'],
+	['name' => 'Settings#feed', 'url' => '/settings/feed', 'verb' => 'POST'],
+	['name' => 'Activities#showList', 'url' => '/', 'verb' => 'GET'],
+	['name' => 'Activities#fetch', 'url' => '/activities/fetch', 'verb' => 'GET'],
+]]);
