@@ -2,10 +2,11 @@
 /**
  * ownCloud
  *
- * @author Sam Tuke, Frank Karlitschek, Bjoern Schiessle
- * @copyright 2012 Sam Tuke <samtuke@owncloud.com>,
- * Frank Karlitschek <frank@owncloud.org>,
- * Bjoern Schiessle <schiessle@owncloud.com>
+ * @copyright (C) 2014 ownCloud, Inc.
+ *
+ * @author Sam Tuke <samtuke@owncloud.com>,
+ * @author Frank Karlitschek <frank@owncloud.org>,
+ * @author Bjoern Schiessle <schiessle@owncloud.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
@@ -22,8 +23,7 @@
  *
  */
 
-namespace OCA\Encryption;
-use OC\User\NoUserException;
+namespace OCA\Files_Encryption;
 
 /**
  * Class for utilities relating to encrypted file storage system
@@ -39,18 +39,73 @@ class Util {
 	const MIGRATION_IN_PROGRESS = -1; // migration is running
 	const MIGRATION_OPEN = 0;         // user still needs to be migrated
 
+	const FILE_TYPE_FILE = 0;
+	const FILE_TYPE_VERSION = 1;
+	const FILE_TYPE_CACHE = 2;
+
+	/**
+	 * @var \OC\Files\View
+	 */
 	private $view; // OC\Files\View object for filesystem operations
+
+	/**
+	 * @var string
+	 */
 	private $userId; // ID of the user we use to encrypt/decrypt files
+
+	/**
+	 * @var string
+	 */
 	private $keyId; // ID of the key we want to manipulate
+
+	/**
+	 * @var bool
+	 */
 	private $client; // Client side encryption mode flag
+
+	/**
+	 * @var string
+	 */
 	private $publicKeyDir; // Dir containing all public user keys
+
+	/**
+	 * @var string
+	 */
 	private $encryptionDir; // Dir containing user's files_encryption
-	private $keyfilesPath; // Dir containing user's keyfiles
-	private $shareKeysPath; // Dir containing env keys for shared files
+
+	/**
+	 * @var string
+	 */
+	private $keysPath; // Dir containing all file related encryption keys
+
+	/**
+	 * @var string
+	 */
 	private $publicKeyPath; // Path to user's public key
+
+	/**
+	 * @var string
+	 */
 	private $privateKeyPath; // Path to user's private key
+
+	/**
+	 * @var string
+	 */
+	private $userFilesDir;
+
+	/**
+	 * @var string
+	 */
 	private $publicShareKeyId;
+
+	/**
+	 * @var string
+	 */
 	private $recoveryKeyId;
+
+	/**
+	 * @var bool
+	 */
 	private $isPublic;
 
 	/**
@@ -73,18 +128,17 @@ class Util {
 		$this->fileFolderName = 'files';
 		$this->userFilesDir =
 				'/' . $userId . '/' . $this->fileFolderName; // TODO: Does this need to be user configurable?
-		$this->publicKeyDir = '/' . 'public-keys';
+		$this->publicKeyDir = Keymanager::getPublicKeyPath();
 		$this->encryptionDir = '/' . $this->userId . '/' . 'files_encryption';
-		$this->keyfilesPath = $this->encryptionDir . '/' . 'keyfiles';
-		$this->shareKeysPath = $this->encryptionDir . '/' . 'share-keys';
+		$this->keysPath = $this->encryptionDir . '/' . 'keys';
 		$this->publicKeyPath =
-				$this->publicKeyDir . '/' . $this->userId . '.public.key'; // e.g. data/public-keys/admin.public.key
+				$this->publicKeyDir . '/' . $this->userId . '.publicKey'; // e.g. data/public-keys/admin.publicKey
 		$this->privateKeyPath =
-				$this->encryptionDir . '/' . $this->userId . '.private.key'; // e.g. data/admin/admin.private.key
+				$this->encryptionDir . '/' . $this->userId . '.privateKey'; // e.g. data/admin/admin.privateKey
 		// make sure that the owners home is mounted
 		\OC\Files\Filesystem::initMountPoints($userId);
 
-		if (\OCA\Encryption\Helper::isPublicAccess()) {
+		if (Helper::isPublicAccess()) {
 			$this->keyId = $this->publicShareKeyId;
 			$this->isPublic = true;
 		} else {
@@ -100,8 +154,7 @@ class Util {
 
 		if (
 			!$this->view->file_exists($this->encryptionDir)
-			or !$this->view->file_exists($this->keyfilesPath)
-			or !$this->view->file_exists($this->shareKeysPath)
+			or !$this->view->file_exists($this->keysPath)
 			or !$this->view->file_exists($this->publicKeyPath)
 			or !$this->view->file_exists($this->privateKeyPath)
 		) {
@@ -125,6 +178,18 @@ class Util {
 		}
 	}
 
+	/**
+	 * create a new public/private key pair for the user
+	 *
+	 * @param string $password password for the private key
+	 */
+	public function replaceUserKeys($password) {
+		$this->backupAllKeys('password_reset');
+		$this->view->unlink($this->publicKeyPath);
+		$this->view->unlink($this->privateKeyPath);
+		$this->setupServerSide($password);
+	}
+
 	/**
 	 * Sets up user folders and keys for serverside encryption
 	 *
@@ -138,8 +203,7 @@ class Util {
 			$this->userDir,
 			$this->publicKeyDir,
 			$this->encryptionDir,
-			$this->keyfilesPath,
-			$this->shareKeysPath
+			$this->keysPath
 		);
 
 		// Check / create all necessary dirs
@@ -216,7 +280,7 @@ class Util {
 	 */
 	public function recoveryEnabledForUser() {
 
-		$recoveryMode = \OC_Preferences::getValue($this->userId, 'files_encryption', 'recovery_enabled', '0');
+		$recoveryMode = \OC::$server->getConfig()->getUserValue($this->userId, 'files_encryption', 'recovery_enabled', '0');
 
 		return ($recoveryMode === '1') ? true : false;
 
@@ -230,7 +294,12 @@ class Util {
 	public function setRecoveryForUser($enabled) {
 
 		$value = $enabled ? '1' : '0';
-		return \OC_Preferences::setValue($this->userId, 'files_encryption', 'recovery_enabled', $value);
+		try {
+			\OC::$server->getConfig()->setUserValue($this->userId, 'files_encryption', 'recovery_enabled', $value);
+			return true;
+		} catch(\OCP\PreConditionNotMetException $e) {
+			return false;
+		}
 
 	}
 
@@ -238,7 +307,7 @@ class Util {
 	 * Find all files and their encryption status within a directory
 	 * @param string $directory The path of the parent directory to search
 	 * @param bool $found the founded files if called again
-	 * @return array keys: plain, encrypted, legacy, broken
+	 * @return array keys: plain, encrypted, broken
 	 * @note $directory needs to be a path relative to OC data dir. e.g.
 	 *       /admin/files NOT /backup OR /home/www/oc/data/admin/files
 	 */
@@ -252,7 +321,6 @@ class Util {
 			$found = array(
 				'plain' => array(),
 				'encrypted' => array(),
-				'legacy' => array(),
 				'broken' => array(),
 			);
 		}
@@ -262,9 +330,13 @@ class Util {
 				while (false !== ($file = readdir($handle))) {
 
 					if ($file !== "." && $file !== "..") {
+						// skip stray part files
+						if (Helper::isPartialFilePath($file)) {
+							continue;
+						}
 
 						$filePath = $directory . '/' . $this->view->getRelativePath('/' . $file);
-						$relPath = \OCA\Encryption\Helper::stripUserFilesPath($filePath);
+						$relPath = Helper::stripUserFilesPath($filePath);
 
 						// If the path is a directory, search
 						// its contents
@@ -306,15 +378,6 @@ class Util {
 									);
 								}
 
-								// If the file uses old
-								// encryption system
-							} elseif (Crypt::isLegacyEncryptedContent($isEncryptedPath, $relPath)) {
-
-								$found['legacy'][] = array(
-									'name' => $file,
-									'path' => $filePath
-								);
-
 								// If the file is not encrypted
 							} else {
 
@@ -396,55 +459,71 @@ class Util {
 			&& $this->isEncryptedPath($path)
 		) {
 
-			$offset = 0;
-			if ($this->containHeader($path)) {
-				$offset = Crypt::BLOCKSIZE;
-			}
+			$cipher = 'AES-128-CFB';
+			$realSize = 0;
 
-			// get the size from filesystem if the file contains a encryption header we
-			// we substract it
-			$size = $this->view->filesize($path) - $offset;
-
-			// fast path, else the calculation for $lastChunkNr is bogus
-			if ($size === 0) {
-				\OC_FileProxy::$enabled = $proxyStatus;
-				return 0;
-			}
-
-			// calculate last chunk nr
-			// next highest is end of chunks, one subtracted is last one
-			// we have to read the last chunk, we can't just calculate it (because of padding etc)
-			$lastChunkNr = ceil($size/ Crypt::BLOCKSIZE) - 1;
-			$lastChunkSize = $size - ($lastChunkNr * Crypt::BLOCKSIZE);
+			// get the size from filesystem
+			$size = $this->view->filesize($path);
 
 			// open stream
-			$stream = fopen('crypt://' . $path, "r");
+			$stream = $this->view->fopen($path, "r");
 
 			if (is_resource($stream)) {
-				// calculate last chunk position
-				$lastChunckPos = ($lastChunkNr * Crypt::BLOCKSIZE);
 
-				// seek to end
-				if (@fseek($stream, $lastChunckPos) === -1) {
-					// storage doesn't support fseek, we need a local copy
-					fclose($stream);
-					$localFile = $this->view->getLocalFile($path);
-					Helper::addTmpFileToMapper($localFile, $path);
-					$stream = fopen('crypt://' . $localFile, "r");
-					if (fseek($stream, $lastChunckPos) === -1) {
-						// if fseek also fails on the local storage, than
-						// there is nothing we can do
-						fclose($stream);
-						\OCP\Util::writeLog('Encryption library', 'couldn\'t determine size of "' . $path, \OCP\Util::ERROR);
-						return $result;
-					}
+				// if the file contains a encryption header we
+				// we set the cipher
+				// and we update the size
+				if ($this->containHeader($path)) {
+					$data = fread($stream,Crypt::BLOCKSIZE);
+					$header = Crypt::parseHeader($data);
+					$cipher = Crypt::getCipher($header);
+					$size -= Crypt::BLOCKSIZE;
 				}
 
+				// fast path, else the calculation for $lastChunkNr is bogus
+				if ($size === 0) {
+					\OC_FileProxy::$enabled = $proxyStatus;
+					return 0;
+				}
+
+				// calculate last chunk nr
+				// next highest is end of chunks, one subtracted is last one
+				// we have to read the last chunk, we can't just calculate it (because of padding etc)
+				$lastChunkNr = ceil($size/Crypt::BLOCKSIZE)-1;
+
+				// calculate last chunk position
+				$lastChunkPos = ($lastChunkNr * Crypt::BLOCKSIZE);
+
 				// get the content of the last chunk
-				$lastChunkContent = fread($stream, $lastChunkSize);
+				if (@fseek($stream, $lastChunkPos, SEEK_CUR) === 0) {
+					$realSize+=$lastChunkNr*6126;
+				}
+				$lastChunkContentEncrypted='';
+				$count=Crypt::BLOCKSIZE;
+				while ($count>0) {
+					$data=fread($stream,Crypt::BLOCKSIZE);
+					$count=strlen($data);
+					$lastChunkContentEncrypted.=$data;
+					if(strlen($lastChunkContentEncrypted)>Crypt::BLOCKSIZE) {
+						$realSize+=6126;
+						$lastChunkContentEncrypted=substr($lastChunkContentEncrypted,Crypt::BLOCKSIZE);
+					}
+				}
+				fclose($stream);
+				$relPath = Helper::stripUserFilesPath($path);
+				$shareKey = Keymanager::getShareKey($this->view, $this->keyId, $this, $relPath);
+				if($shareKey===false) {
+					\OC_FileProxy::$enabled = $proxyStatus;
+					return $result;
+				}
+				$session = new Session($this->view);
+				$privateKey = $session->getPrivateKey();
+				$plainKeyfile = $this->decryptKeyfile($relPath, $privateKey);
+				$plainKey = Crypt::multiKeyDecrypt($plainKeyfile, $shareKey, $privateKey);
+				$lastChunkContent=Crypt::symmetricDecryptFileContent($lastChunkContentEncrypted, $plainKey, $cipher);
 
 				// calc the real file size with the size of the last chunk
-				$realSize = (($lastChunkNr * 6126) + strlen($lastChunkContent));
+				$realSize += strlen($lastChunkContent);
 
 				// store file size
 				$result = $realSize;
@@ -710,8 +789,8 @@ class Util {
 			}
 
 			if ($successful) {
-				$this->view->rename($this->keyfilesPath, $this->keyfilesPath . '.backup');
-				$this->view->rename($this->shareKeysPath, $this->shareKeysPath . '.backup');
+				$this->backupAllKeys('decryptAll', false, false);
+				$this->view->deleteAll($this->keysPath);
 			}
 
 			\OC_FileProxy::$enabled = true;
@@ -723,12 +802,10 @@ class Util {
 	/**
 	 * Encrypt all files in a directory
 	 * @param string $dirPath the directory whose files will be encrypted
-	 * @param null $legacyPassphrase
-	 * @param null $newPassphrase
 	 * @return bool
 	 * @note Encryption is recursive
 	 */
-	public function encryptAll($dirPath, $legacyPassphrase = null, $newPassphrase = null) {
+	public function encryptAll($dirPath) {
 
 		$result = true;
 
@@ -797,42 +874,6 @@ class Util {
 			}
 		}
 
-		// Encrypt legacy encrypted files
-		if (!empty($legacyPassphrase) && !empty($newPassphrase)) {
-
-			foreach ($found['legacy'] as $legacyFile) {
-
-				// Fetch data from file
-				$legacyData = $this->view->file_get_contents($legacyFile['path']);
-
-				// decrypt data, generate catfile
-				$decrypted = Crypt::legacyBlockDecrypt($legacyData, $legacyPassphrase);
-
-				$rawPath = $legacyFile['path'];
-
-				// enable proxy the ensure encryption is handled
-				\OC_FileProxy::$enabled = true;
-
-				// Open enc file handle for binary writing, with same filename as original plain file
-				$encHandle = $this->view->fopen($rawPath, 'wb');
-
-				if (is_resource($encHandle)) {
-
-					// write data to stream
-					fwrite($encHandle, $decrypted);
-
-					// close stream
-					fclose($encHandle);
-				} else {
-					\OCP\Util::writeLog('files_encryption', 'initial encryption: could not encrypt legacy file ' . $rawPath, \OCP\Util::FATAL);
-					$result = false;
-				}
-
-				// disable proxy to prevent file being encrypted twice
-				\OC_FileProxy::$enabled = false;
-			}
-		}
-
 		\OC_FileProxy::$enabled = true;
 
 		if ($versionStatus) {
@@ -866,9 +907,9 @@ class Util {
 
 				break;
 
-			case 'keyfilesPath':
+			case 'keysPath':
 
-				return $this->keyfilesPath;
+				return $this->keysPath;
 
 				break;
 
@@ -904,14 +945,8 @@ class Util {
 		) {
 			return true;
 		}
-		try {
-			$util = new Util($this->view, $user);
-			return $util->ready();
-		} catch (NoUserException $e) {
-			\OCP\Util::writeLog('Encryption library',
-				'No User object for '.$user, \OCP\Util::DEBUG);
-			return false;
-		}
+		$util = new Util($this->view, $user);
+		return $util->ready();
 	}
 
 	/**
@@ -1011,7 +1046,7 @@ class Util {
 			$plainKeyfile = $this->decryptKeyfile($filePath, $privateKey);
 			// Re-enc keyfile to (additional) sharekeys
 			$multiEncKey = Crypt::multiKeyEncrypt($plainKeyfile, $userPubKeys);
-		} catch (Exceptions\EncryptionException $e) {
+		} catch (Exception\EncryptionException $e) {
 			$msg = 'set shareFileKeyFailed (code: ' . $e->getCode() . '): ' . $e->getMessage();
 			\OCP\Util::writeLog('files_encryption', $msg, \OCP\Util::FATAL);
 			return false;
@@ -1064,7 +1099,7 @@ class Util {
 		// Make sure that a share key is generated for the owner too
 		list($owner, $ownerPath) = $this->getUidAndFilename($filePath);
 
-		$ownerPath = \OCA\Encryption\Helper::stripPartialFileExtension($ownerPath);
+		$ownerPath = Helper::stripPartialFileExtension($ownerPath);
 
 		// always add owner to the list of users with access to the file
 		$userIds = array($owner);
@@ -1074,7 +1109,7 @@ class Util {
 			// Find out who, if anyone, is sharing the file
 			$result = \OCP\Share::getUsersSharingFile($ownerPath, $owner);
 			$userIds = \array_merge($userIds, $result['users']);
-			if ($result['public']) {
+			if ($result['public'] || $result['remote']) {
 				$userIds[] = $this->publicShareKeyId;
 			}
 
@@ -1131,7 +1166,12 @@ class Util {
 		// convert to string if preCondition is set
 		$preCondition = ($preCondition === null) ? null : (string)$preCondition;
 
-		return \OC_Preferences::setValue($this->userId, 'files_encryption', 'migration_status', (string)$status, $preCondition);
+		try {
+			\OC::$server->getConfig()->setUserValue($this->userId, 'files_encryption', 'migration_status', (string)$status, $preCondition);
+			return true;
+		} catch(\OCP\PreConditionNotMetException $e) {
+			return false;
+		}
 
 	}
 
@@ -1183,9 +1223,9 @@ class Util {
 
 		$migrationStatus = false;
 		if (\OCP\User::userExists($this->userId)) {
-			$migrationStatus = \OC_Preferences::getValue($this->userId, 'files_encryption', 'migration_status');
+			$migrationStatus = \OC::$server->getConfig()->getUserValue($this->userId, 'files_encryption', 'migration_status', null);
 			if ($migrationStatus === null) {
-				\OC_Preferences::setValue($this->userId, 'files_encryption', 'migration_status', (string)self::MIGRATION_OPEN);
+				\OC::$server->getConfig()->setUserValue($this->userId, 'files_encryption', 'migration_status', (string)self::MIGRATION_OPEN);
 				$migrationStatus = self::MIGRATION_OPEN;
 			}
 		}
@@ -1226,13 +1266,7 @@ class Util {
 
 		// handle public access
 		if ($this->isPublic) {
-			$filename = $path;
-			$fileOwnerUid = $this->userId;
-
-			return array(
-				$fileOwnerUid,
-				$filename
-			);
+			return array($this->userId, $path);
 		} else {
 
 			// Check that UID is valid
@@ -1392,22 +1426,14 @@ class Util {
 	public function checkRecoveryPassword($password) {
 
 		$result = false;
-		$pathKey = '/owncloud_private_key/' . $this->recoveryKeyId . ".private.key";
-
-		$proxyStatus = \OC_FileProxy::$enabled;
-		\OC_FileProxy::$enabled = false;
-
-		$recoveryKey = $this->view->file_get_contents($pathKey);
 
+		$recoveryKey = Keymanager::getPrivateSystemKey($this->recoveryKeyId);
 		$decryptedRecoveryKey = Crypt::decryptPrivateKey($recoveryKey, $password);
 
 		if ($decryptedRecoveryKey) {
 			$result = true;
 		}
 
-		\OC_FileProxy::$enabled = $proxyStatus;
-
-
 		return $result;
 	}
 
@@ -1422,19 +1448,17 @@ class Util {
 	 * add recovery key to all encrypted files
 	 */
 	public function addRecoveryKeys($path = '/') {
-		$dirContent = $this->view->getDirectoryContent($this->keyfilesPath . $path);
+		$dirContent = $this->view->getDirectoryContent($this->keysPath . '/' . $path);
 		foreach ($dirContent as $item) {
 			// get relative path from files_encryption/keyfiles/
-			$filePath = substr($item['path'], strlen('files_encryption/keyfiles'));
-			if ($item['type'] === 'dir') {
+			$filePath = substr($item['path'], strlen('files_encryption/keys'));
+			if ($this->view->is_dir($this->userFilesDir . '/' . $filePath)) {
 				$this->addRecoveryKeys($filePath . '/');
 			} else {
-				$session = new \OCA\Encryption\Session(new \OC\Files\View('/'));
+				$session = new Session(new \OC\Files\View('/'));
 				$sharingEnabled = \OCP\Share::isEnabled();
-				// remove '.key' extension from path e.g. 'file.txt.key' to 'file.txt'
-				$file = substr($filePath, 0, -4);
-				$usersSharing = $this->getSharingUsersArray($sharingEnabled, $file);
-				$this->setSharedFileKeyfiles($session, $usersSharing, $file);
+				$usersSharing = $this->getSharingUsersArray($sharingEnabled, $filePath);
+				$this->setSharedFileKeyfiles($session, $usersSharing, $filePath);
 			}
 		}
 	}
@@ -1443,16 +1467,14 @@ class Util {
 	 * remove recovery key to all encrypted files
 	 */
 	public function removeRecoveryKeys($path = '/') {
-		$dirContent = $this->view->getDirectoryContent($this->keyfilesPath . $path);
+		$dirContent = $this->view->getDirectoryContent($this->keysPath . '/' . $path);
 		foreach ($dirContent as $item) {
 			// get relative path from files_encryption/keyfiles
-			$filePath = substr($item['path'], strlen('files_encryption/keyfiles'));
-			if ($item['type'] === 'dir') {
+			$filePath = substr($item['path'], strlen('files_encryption/keys'));
+			if ($this->view->is_dir($this->userFilesDir . '/' . $filePath)) {
 				$this->removeRecoveryKeys($filePath . '/');
 			} else {
-				// remove '.key' extension from path e.g. 'file.txt.key' to 'file.txt'
-				$file = substr($filePath, 0, -4);
-				$this->view->unlink($this->shareKeysPath . '/' . $file . '.' . $this->recoveryKeyId . '.shareKey');
+				$this->view->unlink($this->keysPath . '/' . $filePath . '/' . $this->recoveryKeyId . '.shareKey');
 			}
 		}
 	}
@@ -1482,27 +1504,17 @@ class Util {
 		}
 		$filteredUids = $this->filterShareReadyUsers($userIds);
 
-		$proxyStatus = \OC_FileProxy::$enabled;
-		\OC_FileProxy::$enabled = false;
-
 		//decrypt file key
-		$encKeyfile = $this->view->file_get_contents($this->keyfilesPath . $file . ".key");
-		$shareKey = $this->view->file_get_contents(
-			$this->shareKeysPath . $file . "." . $this->recoveryKeyId . ".shareKey");
+		$encKeyfile = Keymanager::getFileKey($this->view, $this, $file);
+		$shareKey = Keymanager::getShareKey($this->view, $this->recoveryKeyId, $this, $file);
 		$plainKeyfile = Crypt::multiKeyDecrypt($encKeyfile, $shareKey, $privateKey);
 		// encrypt file key again to all users, this time with the new public key for the recovered use
 		$userPubKeys = Keymanager::getPublicKeys($this->view, $filteredUids['ready']);
 		$multiEncKey = Crypt::multiKeyEncrypt($plainKeyfile, $userPubKeys);
 
-		// write new keys to filesystem TDOO!
-		$this->view->file_put_contents($this->keyfilesPath . $file . '.key', $multiEncKey['data']);
-		foreach ($multiEncKey['keys'] as $userId => $shareKey) {
-			$shareKeyPath = $this->shareKeysPath . $file . '.' . $userId . '.shareKey';
-			$this->view->file_put_contents($shareKeyPath, $shareKey);
-		}
+		Keymanager::setFileKey($this->view, $this, $file, $multiEncKey['data']);
+		Keymanager::setShareKeys($this->view, $this, $file, $multiEncKey['keys']);
 
-		// Return proxy to original status
-		\OC_FileProxy::$enabled = $proxyStatus;
 	}
 
 	/**
@@ -1511,16 +1523,14 @@ class Util {
 	 * @param string $privateKey private recovery key which is used to decrypt the files
 	 */
 	private function recoverAllFiles($path, $privateKey) {
-		$dirContent = $this->view->getDirectoryContent($this->keyfilesPath . $path);
+		$dirContent = $this->view->getDirectoryContent($this->keysPath . '/' . $path);
 		foreach ($dirContent as $item) {
 			// get relative path from files_encryption/keyfiles
-			$filePath = substr($item['path'], strlen('files_encryption/keyfiles'));
-			if ($item['type'] === 'dir') {
+			$filePath = substr($item['path'], strlen('files_encryption/keys'));
+			if ($this->view->is_dir($this->userFilesDir . '/' . $filePath)) {
 				$this->recoverAllFiles($filePath . '/', $privateKey);
 			} else {
-				// remove '.key' extension from path e.g. 'file.txt.key' to 'file.txt'
-				$file = substr($filePath, 0, -4);
-				$this->recoverFile($file, $privateKey);
+				$this->recoverFile($filePath, $privateKey);
 			}
 		}
 	}
@@ -1531,33 +1541,70 @@ class Util {
 	 */
 	public function recoverUsersFiles($recoveryPassword) {
 
-		// Disable encryption proxy to prevent recursive calls
-		$proxyStatus = \OC_FileProxy::$enabled;
-		\OC_FileProxy::$enabled = false;
-
-		$encryptedKey = $this->view->file_get_contents(
-			'/owncloud_private_key/' . $this->recoveryKeyId . '.private.key');
+		$encryptedKey = Keymanager::getPrivateSystemKey( $this->recoveryKeyId);
 		$privateKey = Crypt::decryptPrivateKey($encryptedKey, $recoveryPassword);
 
-		\OC_FileProxy::$enabled = $proxyStatus;
-
 		$this->recoverAllFiles('/', $privateKey);
 	}
 
 	/**
 	 * create a backup of all keys from the user
 	 *
-	 * @param string $purpose (optional) define the purpose of the backup, will be part of the backup folder
+	 * @param string $purpose define the purpose of the backup, will be part of the backup folder name
+	 * @param boolean $timestamp (optional) should a timestamp be added, default true
+	 * @param boolean $includeUserKeys (optional) include users private-/public-key, default true
 	 */
-	public function backupAllKeys($purpose = '') {
+	public function backupAllKeys($purpose, $timestamp = true, $includeUserKeys = true) {
 		$this->userId;
-		$backupDir = $this->encryptionDir . '/backup.';
-		$backupDir .= ($purpose === '') ? date("Y-m-d_H-i-s") . '/' : $purpose . '.' . date("Y-m-d_H-i-s") . '/';
+		$backupDir = $this->encryptionDir . '/backup.' . $purpose;
+		$backupDir .= ($timestamp) ? '.' . date("Y-m-d_H-i-s") . '/' : '/';
 		$this->view->mkdir($backupDir);
-		$this->view->copy($this->shareKeysPath, $backupDir . 'share-keys/');
-		$this->view->copy($this->keyfilesPath, $backupDir . 'keyfiles/');
-		$this->view->copy($this->privateKeyPath, $backupDir . $this->userId . '.private.key');
-		$this->view->copy($this->publicKeyPath, $backupDir . $this->userId . '.public.key');
+		$this->view->copy($this->keysPath, $backupDir . 'keys/');
+		if ($includeUserKeys) {
+			$this->view->copy($this->privateKeyPath, $backupDir . $this->userId . '.privateKey');
+			$this->view->copy($this->publicKeyPath, $backupDir . $this->userId . '.publicKey');
+		}
+	}
+
+	/**
+	 * restore backup
+	 *
+	 * @param string $backup complete name of the backup
+	 * @return boolean
+	 */
+	public function restoreBackup($backup) {
+		$backupDir = $this->encryptionDir . '/backup.' . $backup . '/';
+
+		$fileKeysRestored = $this->view->rename($backupDir . 'keys', $this->encryptionDir . '/keys');
+
+		$pubKeyRestored = $privKeyRestored = true;
+		if (
+			$this->view->file_exists($backupDir . $this->userId . '.privateKey') &&
+			$this->view->file_exists($backupDir . $this->userId . '.privateKey')
+		) {
+
+			$pubKeyRestored = $this->view->rename($backupDir . $this->userId . '.publicKey', $this->publicKeyPath);
+			$privKeyRestored = $this->view->rename($backupDir . $this->userId . '.privateKey', $this->privateKeyPath);
+		}
+
+		if ($fileKeysRestored && $pubKeyRestored && $privKeyRestored) {
+			$this->view->deleteAll($backupDir);
+
+			return true;
+		}
+
+		return false;
+	}
+
+	/**
+	 * delete backup
+	 *
+	 * @param string $backup complete name of the backup
+	 * @return boolean
+	 */
+	public function deleteBackup($backup) {
+		$backupDir = $this->encryptionDir . '/backup.' . $backup . '/';
+		return $this->view->deleteAll($backupDir);
 	}
 
 	/**
@@ -1610,14 +1657,17 @@ class Util {
 	 */
 	public function initEncryption($params) {
 
-		$session = new \OCA\Encryption\Session($this->view);
+		$session = new Session($this->view);
 
 		// we tried to initialize the encryption app for this session
-		$session->setInitialized(\OCA\Encryption\Session::INIT_EXECUTED);
+		$session->setInitialized(Session::INIT_EXECUTED);
 
 		$encryptedKey = Keymanager::getPrivateKey($this->view, $params['uid']);
 
-		$privateKey = Crypt::decryptPrivateKey($encryptedKey, $params['password']);
+		$privateKey = false;
+		if ($encryptedKey) {
+			$privateKey = Crypt::decryptPrivateKey($encryptedKey, $params['password']);
+		}
 
 		if ($privateKey === false) {
 			\OCP\Util::writeLog('Encryption library', 'Private key for user "' . $params['uid']
@@ -1626,7 +1676,7 @@ class Util {
 		}
 
 		$session->setPrivateKey($privateKey);
-		$session->setInitialized(\OCA\Encryption\Session::INIT_SUCCESSFUL);
+		$session->setInitialized(Session::INIT_SUCCESSFUL);
 
 		return $session;
 	}
@@ -1635,7 +1685,7 @@ class Util {
 	 * remove encryption related keys from the session
 	 */
 	public function closeEncryptionSession() {
-		$session = new \OCA\Encryption\Session($this->view);
+		$session = new Session($this->view);
 		$session->closeSession();
 	}
 
