@@ -26,15 +26,27 @@
 namespace OCA\user_ldap;
 
 use OCA\user_ldap\lib\BackendUtility;
+use OCA\user_ldap\lib\Access;
 use OCA\user_ldap\lib\user\OfflineUser;
 use OCA\User_LDAP\lib\User\User;
+use OCP\IConfig;
 
-class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
-	/**
-	 * @var string[] $homesToKill
-	 */
+class USER_LDAP extends BackendUtility implements \OCP\IUserBackend, \OCP\UserInterface {
+	/** @var string[] $homesToKill */
 	protected $homesToKill = array();
 
+	/** @var \OCP\IConfig */
+	protected $ocConfig;
+
+	/**
+	 * @param \OCA\user_ldap\lib\Access $access
+	 * @param \OCP\IConfig $ocConfig
+	 */
+	public function __construct(Access $access, IConfig $ocConfig) {
+		parent::__construct($access);
+		$this->ocConfig = $ocConfig;
+	}
+
 	/**
 	 * checks whether the user is allowed to change his avatar in ownCloud
 	 * @param string $uid the ownCloud user name
@@ -56,7 +68,7 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 	 * Check if the password is correct
 	 * @param string $uid The username
 	 * @param string $password The password
-	 * @return boolean
+	 * @return false|string
 	 *
 	 * Check if the password is correct without logging in the user
 	 */
@@ -176,7 +188,6 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 	 * check if a user exists
 	 * @param string $uid the username
 	 * @return boolean
-	 * @throws \Exception when connection could not be established
 	 */
 	public function userExists($uid) {
 		if($this->access->connection->isCached('userExists'.$uid)) {
@@ -184,7 +195,6 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 		}
 		//getting dn, if false the user does not exist. If dn, he may be mapped only, requires more checking.
 		$user = $this->access->userManager->get($uid);
-
 		if(is_null($user)) {
 			\OCP\Util::writeLog('user_ldap', 'No DN found for '.$uid.' on '.
 				$this->access->connection->ldapHost, \OCP\Util::DEBUG);
@@ -196,12 +206,17 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 			return true;
 		}
 
-		$result = $this->userExistsOnLDAP($user);
-		$this->access->connection->writeToCache('userExists'.$uid, $result);
-		if($result === true) {
-			$user->update();
+		try {
+			$result = $this->userExistsOnLDAP($user);
+			$this->access->connection->writeToCache('userExists'.$uid, $result);
+			if($result === true) {
+				$user->update();
+			}
+			return $result;
+		} catch (\Exception $e) {
+			\OCP\Util::writeLog('user_ldap', $e->getMessage(), \OCP\Util::WARN);
+			return false;
 		}
-		return $result;
 	}
 
 	/**
@@ -211,8 +226,7 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 	* @return bool
 	*/
 	public function deleteUser($uid) {
-		$pref = \OC::$server->getConfig();
-		$marked = $pref->getUserValue($uid, 'user_ldap', 'isDeleted', 0);
+		$marked = $this->ocConfig->getUserValue($uid, 'user_ldap', 'isDeleted', 0);
 		if(intval($marked) === 0) {
 			\OC::$server->getLogger()->notice(
 				'User '.$uid . ' is not marked as deleted, not cleaning up.',
@@ -224,9 +238,9 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 
 		//Get Home Directory out of user preferences so we can return it later,
 		//necessary for removing directories as done by OC_User.
-		$home = $pref->getUserValue($uid, 'user_ldap', 'homePath', '');
+		$home = $this->ocConfig->getUserValue($uid, 'user_ldap', 'homePath', '');
 		$this->homesToKill[$uid] = $home;
-		$this->access->unmapUser($uid);
+		$this->access->getUserMapper()->unmap($uid);
 
 		return true;
 	}
@@ -237,21 +251,20 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 	* @return string|bool
 	*/
 	public function getHome($uid) {
-		if(isset($this->homesToKill[$uid]) && !empty($this->homesToKill[$uid])) {
-			//a deleted user who needs some clean up
-			return $this->homesToKill[$uid];
-		}
-
 		// user Exists check required as it is not done in user proxy!
 		if(!$this->userExists($uid)) {
 			return false;
 		}
 
+		if(isset($this->homesToKill[$uid]) && !empty($this->homesToKill[$uid])) {
+			//a deleted user who needs some clean up
+			return $this->homesToKill[$uid];
+		}
+
 		$cacheKey = 'getHome'.$uid;
 		if($this->access->connection->isCached($cacheKey)) {
 			return $this->access->connection->getFromCache($cacheKey);
 		}
-		$pref = \OC::$server->getConfig();
 		if(strpos($this->access->connection->homeFolderNamingRule, 'attr:') === 0) {
 			$attr = substr($this->access->connection->homeFolderNamingRule, strlen('attr:'));
 			$homedir = $this->access->readAttribute(
@@ -267,13 +280,15 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 				) {
 					$homedir = $path;
 				} else {
-					$homedir = \OCP\Config::getSystemValue('datadirectory',
+					$homedir = $this->ocConfig->getSystemValue('datadirectory',
 						\OC::$SERVERROOT.'/data' ) . '/' . $homedir[0];
 				}
 				$this->access->connection->writeToCache($cacheKey, $homedir);
 				//we need it to store it in the DB as well in case a user gets
 				//deleted so we can clean up afterwards
-				$pref->setUserValue($uid, 'user_ldap', 'homePath', $homedir);
+				$this->ocConfig->setUserValue(
+					$uid, 'user_ldap', 'homePath', $homedir
+				);
 				//TODO: if home directory changes, the old one needs to be removed.
 				return $homedir;
 			}
@@ -281,7 +296,7 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 
 		//false will apply default behaviour as defined and done by OC_User
 		$this->access->connection->writeToCache($cacheKey, false);
-		$pref->setUserValue($uid, 'user_ldap', 'homePath', '');
+		$this->ocConfig->setUserValue($uid, 'user_ldap', 'homePath', '');
 		return false;
 	}
 
@@ -363,8 +378,7 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 	 * @return int|bool
 	 */
 	public function countUsers() {
-		$filter = \OCP\Util::mb_str_replace(
-			'%uid', '*', $this->access->connection->ldapLoginFilter, 'UTF-8');
+		$filter = $this->access->getFilterForUserCount();
 		$cacheKey = 'countUsers-'.$filter;
 		if(!is_null($entries = $this->access->connection->getFromCache($cacheKey))) {
 			return $entries;
@@ -373,4 +387,13 @@ class USER_LDAP extends BackendUtility implements \OCP\UserInterface {
 		$this->access->connection->writeToCache($cacheKey, $entries);
 		return $entries;
 	}
+
+	/**
+	 * Backend name to be shown in user management
+	 * @return string the name of the backend to be shown
+	 */
+	public function getBackendName(){
+		return 'LDAP';
+	}
+
 }
